<?php
/*
Plugin Name: Enhanced Categories
Version: 1.0
Plugin URI: http://enhanced-categories.vincentprat.info
Description: Allows to get better control over the category listing. Please make a donation if you are satisfied.
Author: Vincent Prat
Author URI: http://www.vincentprat.info
*/

/*  Copyright 2006 Vincent Prat

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Version of the plugin
define('ENHANCED_CATEGORIES_CURRENT_VERSION', '1.0' );

// i18n plugin domain 
define('ENHANCED_CATEGORIES_I18N_DOMAIN', 'enhanced-categories');

// Options
define('ENHANCED_CATEGORIES_VERSION_OPTION', 'enh_cats_version');
define('ENHANCED_CATEGORIES_SHOW_SYMBOL_OPTION', 'enh_cats_show_symbol');
define('ENHANCED_CATEGORIES_NO_CHILD_SYMBOL_OPTION', 'enh_cats_no_child_symbol');
define('ENHANCED_CATEGORIES_HIDE_SYMBOL_OPTION', 'enh_cats_hide_symbol');
define('ENHANCED_CATEGORIES_EFFECT_OPTION', 'enh_cats_effect');
define('ENHANCED_CATEGORIES_SHOW_COUNT_OPTION', 'enh_cats_show_count');
define('ENHANCED_CATEGORIES_SHOW_RSS_OPTION', 'enh_cats_show_rss');
define('ENHANCED_CATEGORIES_HIDE_EMPTY_OPTION', 'enh_cats_hide_empty');

/**
 * Initialise the internationalisation domain
 */
$enh_cats_is_i18n_setup = false;
function enh_cats_init_i18n() {
	global $enh_cats_is_i18n_setup;

	if ($enh_cats_is_i18n_setup == false) {
		load_plugin_textdomain(ENHANCED_CATEGORIES_I18N_DOMAIN, 'wp-content/plugins/enhanced-categories');
		$enh_cats_is_i18n_setup = true;
	}
}

/**
 * Plugin activation
 */
add_action('activate_enhanced-categories/enhanced-categories.php','enh_cats_plugin_activation');
function enh_cats_plugin_activation() {
	$installed_version = enh_cats_get_installed_version();
	
	if ( $installed_version==enh_cats_get_current_version() ) {
		// do nothing
	} else if ( $installed_version=='' ) {
		// Add all options, nothing already installed
		add_option(
			ENHANCED_CATEGORIES_SHOW_SYMBOL_OPTION,
			'<span style="color: blue;">&raquo;</span>',
			'The symbol to display when the category can be shown.' );
		add_option(
			ENHANCED_CATEGORIES_HIDE_SYMBOL_OPTION,
			'<span style="color: red;">&laquo;</span>',
			'The symbol to display when the category can be hidden.' );
		add_option(
			ENHANCED_CATEGORIES_NO_CHILD_SYMBOL_OPTION,
			'<span style="color: white;">&raquo;</span>',
			'The symbol to display when the category has no child.' );
		add_option(
			ENHANCED_CATEGORIES_EFFECT_OPTION,
			'none',
			'The effect used to show/hide the categories.' );
		add_option(
			ENHANCED_CATEGORIES_SHOW_COUNT_OPTION,
			'0',
			'The effect used to show/hide the number of posts.' );
		add_option(
			ENHANCED_CATEGORIES_SHOW_RSS_OPTION,
			'0',
			'The effect used to show/hide the rss link.' );
		add_option(
			ENHANCED_CATEGORIES_HIDE_EMPTY_OPTION,
			'0',
			'The effect used to hide empty categories.' );
		add_option( 
			ENHANCED_CATEGORIES_VERSION_OPTION, 
			enh_cats_get_current_version(),
			'Enhanced Categories version number');	
	}
	
	// Update version number
	update_option( ENHANCED_CATEGORIES_VERSION_OPTION, enh_cats_get_current_version() );	
}

/**
 * Add options page
 */
add_action( 'admin_menu', 'enh_cats_add_pages' );
function enh_cats_add_pages() {
	enh_cats_init_i18n();

	add_options_page( __('Enhanced Categories', ENHANCED_CATEGORIES_I18N_DOMAIN), 
		__('Enhanced Categories', ENHANCED_CATEGORIES_I18N_DOMAIN), 
		8, 
		'enhanced-categories/enhanced-categories_options_page.php' );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_version'
 */
function enh_cats_get_installed_version() {
	return get_option( ENHANCED_CATEGORIES_VERSION_OPTION );
}

/**
 * Wrapper for the defined constant ENHANCED_CATEGORIES_CURRENT_VERSION
 */
function enh_cats_get_current_version() {	
	return ENHANCED_CATEGORIES_CURRENT_VERSION;
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_NO_CHILD_SYMBOL_OPTION'
 */
function enh_cats_get_no_child_symbol() {
	return str_replace("\\'", "'", get_option( ENHANCED_CATEGORIES_NO_CHILD_SYMBOL_OPTION ));
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_NO_CHILD_SYMBOL_OPTION'
 */
function enh_cats_set_no_child_symbol($value) {
	return update_option( ENHANCED_CATEGORIES_NO_CHILD_SYMBOL_OPTION, $value );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_SHOW_SYMBOL_OPTION'
 */
function enh_cats_get_show_symbol() {
	return str_replace("\\'", "'", get_option( ENHANCED_CATEGORIES_SHOW_SYMBOL_OPTION ));
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_SHOW_SYMBOL_OPTION'
 */
function enh_cats_set_show_symbol($value) {
	return update_option( ENHANCED_CATEGORIES_SHOW_SYMBOL_OPTION, $value );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_HIDE_SYMBOL_OPTION'
 */
function enh_cats_get_hide_symbol() {
	return str_replace("\\'", "'", get_option( ENHANCED_CATEGORIES_HIDE_SYMBOL_OPTION ));
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_HIDE_SYMBOL_OPTION'
 */
function enh_cats_set_hide_symbol($value) {
	return update_option( ENHANCED_CATEGORIES_HIDE_SYMBOL_OPTION, $value );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_EFFECT_OPTION'
 */
function enh_cats_get_effect() {
	return get_option( ENHANCED_CATEGORIES_EFFECT_OPTION );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_EFFECT_OPTION'
 */
function enh_cats_set_effect($value) {
	return update_option( ENHANCED_CATEGORIES_EFFECT_OPTION, $value );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_SHOW_COUNT_OPTION'
 */
function enh_cats_get_show_count() {
	return get_option( ENHANCED_CATEGORIES_SHOW_COUNT_OPTION );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_SHOW_COUNT_OPTION'
 */
function enh_cats_set_show_count($value) {
	return update_option( ENHANCED_CATEGORIES_SHOW_COUNT_OPTION, $value );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_SHOW_RSS_OPTION'
 */
function enh_cats_get_show_rss() {
	return get_option( ENHANCED_CATEGORIES_SHOW_RSS_OPTION );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_SHOW_RSS_OPTION'
 */
function enh_cats_set_show_rss($value) {
	return update_option( ENHANCED_CATEGORIES_SHOW_RSS_OPTION, $value );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_HIDE_EMPTY_OPTION'
 */
function enh_cats_get_hide_empty() {
	return get_option( ENHANCED_CATEGORIES_HIDE_EMPTY_OPTION );
}

/**
 * Wrapper for the option 'ENHANCED_CATEGORIES_HIDE_EMPTY_OPTION'
 */
function enh_cats_set_hide_empty($value) {
	return update_option( ENHANCED_CATEGORIES_HIDE_EMPTY_OPTION, $value );
}

/**
* Class that outputs the category listing
*/
class Walker_EnhancedCategories extends Walker {
	var $tree_type = 'category';
	var $db_fields = array ('parent' => 'parent', 'id' => 'term_id'); //TODO: decouple this

	function start_lvl($output, $depth, $args) {
		return $output;
	}

	function end_lvl($output, $depth, $args) {
		return $output;
	}

	function start_el($output, $category, $depth, $args) {
		extract($args);

		if (enh_cats_category_has_children($category->term_id)) {
			$button  = '<a id="enh_cats_categoryButton'. $category->term_id . '" '; 
			$button .= 'href="javascript:enh_cats_expandCategory(' . $category->term_id . ');">' . enh_cats_get_show_symbol() . '</a> &nbsp; ';
		} else {
			$button = enh_cats_get_no_child_symbol() . ' &nbsp; ';
		}
		
		$cat_name = attribute_escape( $category->name);
		$cat_name = apply_filters( 'list_cats', $cat_name, $category );
		$link = '<a href="' . get_category_link( $category->term_id ) . '" ';
		if ( $use_desc_for_title == 0 || empty($category->description) )
			$link .= 'title="' . sprintf(__( 'View all posts filed under %s' ), $cat_name) . '"';
		else
			$link .= 'title="' . attribute_escape( apply_filters( 'category_description', $category->description, $category )) . '"';
		$link .= '>';
		$link .= $cat_name . '</a>';

		if ( (! empty($feed_image)) || (! empty($feed)) ) {
			$link .= ' ';

			if ( empty($feed_image) )
				$link .= '(';

			$link .= '<a href="' . get_category_rss_link( 0, $category->term_id, $category->slug ) . '"';

			if ( empty($feed) )
				$alt = ' alt="' . sprintf(__( 'Feed for all posts filed under %s' ), $cat_name ) . '"';
			else {
				$title = ' title="' . $feed . '"';
				$alt = ' alt="' . $feed . '"';
				$name = $feed;
				$link .= $title;
			}

			$link .= '>';

			if ( empty($feed_image) )
				$link .= $name;
			else
				$link .= "<img src='$feed_image'$alt$title" . ' />';
			$link .= '</a>';
			if ( empty($feed_image) )
				$link .= ')';
		}

		if ( isset($show_count) && $show_count )
			$link .= ' (' . intval($category->count) . ')';

		if ( isset($show_date) && $show_date ) {
			$link .= ' ' . gmdate('Y-m-d', $category->last_update_timestamp);
		}

		$output .= "\t<h2>$button $link</h2>";
		$output .= "<div id='enh_cats_categoryContent" . $category->term_id ."' style='display: none; margin-left: 10px;'>";
		$output .= "\n";

		return $output;
	}

	function end_el($output, $page, $depth, $args) {
		$output .= "</div>\n";
		return $output;
	}

}

/**
 * Get the list of the categories 
 */ 
function enh_cats_get_categories($args='') {
	$defaults = array(
		'show_option_all' => '', 
		'orderby' => 'name',
		'order' => 'ASC', 
		'show_last_update' => 0,
		'show_count' => enh_cats_get_show_count(),
		'hide_empty' => enh_cats_get_hide_empty(), 
		'use_desc_for_title' => 1,
		'child_of' => 0, 
		'feed' => enh_cats_get_show_rss()==1 ? 'RSS' : '',
		'feed_image' => '', 
		'exclude' => '',
		'hierarchical' => true
	);

	$r = wp_parse_args( $args, $defaults );
	
	return get_categories($r);
}

/**
* see if a category has children
*/
function enh_cats_category_has_children($id) {
	global $enh_cats_global_category_cache;
	foreach ($enh_cats_global_category_cache as $cat) {	
		if ($cat->parent == $id) {
			return true;
		}
	}
	return false;
}

/**
 * Function to insert the javascript in the page header
 */
add_action('wp_head', 'enh_cats_insert_javascript');
$enh_cats_global_category_cache = enh_cats_get_categories();

function enh_cats_insert_javascript() {
	global $enh_cats_global_category_cache;
?>
<!-- Start Of Script Generated By Enhanced Categories -->
<script type="text/javascript">
	var enh_cats_js_categories = new Array(<?php echo count($enh_cats_global_category_cache); ?>);
<?php
	// Declare the JS variables
	$i = 0;
	foreach ($enh_cats_global_category_cache as $cat) {	
		$clean_cat_name = str_replace("'","\'",$cat->name);
		echo "\tenh_cats_js_categories[$i] = new Array($cat->term_id, '$clean_cat_name', true);\n";
		$i++;
	}
?>
	function enh_cats_showContent(content, effect) {
		if (effect=='jQuery') {
			jQuery(content).slideDown();
		} else if (effect=='scriptaculous') {
			new Effect.BlindDown(content);
		} else {
			content.style.display = 'block';
		}
	}
	
	function enh_cats_hideContent(content, effect) {
		if (effect=='jQuery') {
			jQuery(content).slideUp();
		} else if (effect=='scriptaculous') {
			new Effect.BlindUp(content);
		} else {
			content.style.display = 'none';
		}
	}

	function enh_cats_expandCategory(categoryId) {
		var showSymbol = <?php echo "'" . str_replace("'","\'",enh_cats_get_show_symbol()) . "'"; ?>;
		var hideSymbol = <?php echo "'" . str_replace("'","\'",enh_cats_get_hide_symbol()) . "'"; ?>;
		var effect = <?php echo "'" . enh_cats_get_effect() . "'"; ?>;
		
		for (var i=0; i<enh_cats_js_categories.length; i++) {
			var currentId = enh_cats_js_categories[i][0];
			var currentName = enh_cats_js_categories[i][1];
			var currentlyHidden = enh_cats_js_categories[i][2];
			var currentButton = document.getElementById('enh_cats_categoryButton' + currentId);
			var currentContent = document.getElementById('enh_cats_categoryContent' + currentId);
			if (currentId==categoryId) {
				// Expand this category	or contract it if it was expanded before
				if (currentlyHidden) {
					enh_cats_showContent(currentContent, effect);
					currentButton.innerHTML = hideSymbol;
					enh_cats_js_categories[i][2] = false;
				} else {
					enh_cats_hideContent(currentContent, effect);
					currentButton.innerHTML = showSymbol;
					enh_cats_js_categories[i][2] = true;
				} 
			} 
		}
	}
</script>
<!-- End Of Script Generated By Enhanced Categories -->
<?php
}

/**
 * Function to insert the html code for the link categories
 */
function enh_cats_insert_categories($args = '') {
	global $enh_cats_global_category_cache;
	
	$defaults = array(
		'show_option_all' => '', 
		'orderby' => 'name',
		'order' => 'ASC', 
		'show_last_update' => 0,
		'show_count' => enh_cats_get_show_count(),
		'hide_empty' => enh_cats_get_hide_empty(), 
		'use_desc_for_title' => 1,
		'child_of' => 0, 
		'feed' => enh_cats_get_show_rss()==1 ? 'RSS' : '',
		'feed_image' => '', 
		'exclude' => '',
		'hierarchical' => true
	);

	$r = wp_parse_args( $args, $defaults );
	
	if ( !isset( $r['pad_counts'] ) && $r['show_count'] && $r['hierarchical'] ) {
		$r['pad_counts'] = true;
	}

	if ( isset( $r['show_date'] ) ) {
		$r['include_last_update_time'] = $r['show_date'];
	}

	extract( $r );

	$categories = get_categories($r);

	$output = '';
	$output .= '<li class="categories">';

	if ( empty($categories) ) {
		$output .= '<li>' . __("No categories") . '</li>';
	} else {
		global $wp_query;

		if( !empty($show_option_all) )
			$output .= '<li><a href="' .  get_bloginfo('url')  . '">' . $show_option_all . '</a></li>';

		if ( is_category() )
			$r['current_category'] = $wp_query->get_queried_object_id();

		if ( $hierarchical )
			$depth = 0;  // Walk the full depth.
		else
			$depth = -1; // Flat.

		$output .= enh_cats_walk_category_tree($categories, $depth, $r);
	}

	$output .= '</li>';

	$output = apply_filters('wp_list_categories', $output);
	
	echo $output;
}

function enh_cats_walk_category_tree() {
	$walker = new Walker_EnhancedCategories;
	$args = func_get_args();
	return call_user_func_array(array(&$walker, 'walk'), $args);
}
?>